@@ -1127,13 +1127,14 @@ sub _number_format
     my @args = @_;
     my( $num, $tho, $dec, $prec ) = @args;
     $self->_load_class( 'Module::Generic::Number' ) || return( $self->pass_error );
-    my $fmt = Module::Generic::Number->new( $num,
-        thousands   => $tho,
-        decimal     => $dec,
-        precision   => $prec,
-    );
+    my $fmt = Module::Generic::Number->new( $num );
     # 1 means with trailing zeros
-    return( $fmt->format( precision => $prec, decimal_fill => 1 ) );
+    return( $fmt->format(
+        thousand => $tho,
+        decimal  => $dec,
+        precision => $prec,
+        decimal_fill => 1,
+    ));
 }
 
 sub _placeholder_regexp { return( $PLACEHOLDER_REGEXP ) }
