@@ -230,8 +230,9 @@ sub execute
         ( $el = $q->elements ) &&
         $el->types->length )
     {
+        $self->messagec( 5, "There are {green}", $el->length, "{/} elements for this query." );
         my $types = $q->binded_types_as_param;
-        $self->messagec( 5, "Using {green}", $el->types->length, "{/} binded types from the query object -> ", sub{ $self->Module::Generic::dump( $types ) } );
+        $self->messagec( 5, "Using {green}", $el->types->length, "{/} binded types from the query object ($q) -> ", sub{ $self->Module::Generic::dump( $types ) } );
         @binded_types = @$types;
     }
     
